# Copyright (c) 2018, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import unittest
from datetime import datetime

import logging

from geopathfinder.naming_conventions.eodr_naming import eoDRFilename, create_eodr_filename

logging.basicConfig(level=logging.INFO)


class TesteoDRFilename(unittest.TestCase):

    def setUp(self):
        self.id_1 = "123456"
        self.id_2 = "654321"
        self.dt_1 = datetime(2008, 1, 1, 12, 23, 33)
        self.dt_2 = datetime(2009, 2, 2, 22, 1, 1)

        fields_1 = {'id': self.id_1, 'dt_1': self.dt_1, 'band': 'B1'}

        self.eodr_fn_1 = eoDRFilename(fields_1, convert=True)

        fields_2 = {'id': self.id_2, 'dt_1':  self.dt_1, 'dt_2': self.dt_2, 'band': 'B12', 'd1': '45'}

        self.eodr_fn_2 = eoDRFilename(fields_2, convert=True)

        fn = '123456------_20181220T232333_---------------_B5_34_aug.vrt'
        self.eodr_fn_3 = create_eodr_filename(fn, convert=True)

        self.eodr_fn_4 = create_eodr_filename(fn)

    def test_build_eodr_filename(self):
        """
        Test building SGRT file name.

        """
        fn = ('123456------_20080101T122333_---------------_B1.vrt')

        self.assertEqual(str(self.eodr_fn_1), fn)

    def test_get_n_set_date(self):
        """
        Test set and get start and end date.

        """
        self.assertEqual(self.eodr_fn_2['dt_1'].date(), self.dt_1.date())
        self.assertEqual(self.eodr_fn_2['dt_2'].date(), self.dt_2.date())

        new_start_time = datetime(2009, 1, 1, 12, 23, 33)
        self.eodr_fn_2['dt_1'] = new_start_time

        self.assertEqual(self.eodr_fn_2['dt_1'].date(), new_start_time.date())

    def test_create_eodr_filename(self):
        """
        Tests the creation of a SmartFilename from a given string filename.

        """

        # testing for decoded types
        self.assertEqual(self.eodr_fn_3.get_field('id'), '123456')
        self.assertEqual(self.eodr_fn_3.get_field('dt_1'), datetime(2018, 12, 20, 23, 23, 33))
        self.assertEqual(self.eodr_fn_3.get_field('dt_2'), '')
        self.assertEqual(self.eodr_fn_3.get_field('band'), 'B5')
        self.assertEqual(self.eodr_fn_3.get_field('d1'), '34')
        self.assertEqual(self.eodr_fn_3.get_field('d2'), 'aug')
        self.assertEqual(self.eodr_fn_3.ext, '.vrt')

        # testing for string types
        self.assertEqual(self.eodr_fn_4.get_field('id'), '123456')
        self.assertEqual(self.eodr_fn_4.get_field('dt_1'), '20181220T232333')
        self.assertEqual(self.eodr_fn_4.get_field('dt_2'), '')
        self.assertEqual(self.eodr_fn_4.get_field('band'), 'B5')
        self.assertEqual(self.eodr_fn_4.get_field('d1'), '34')
        self.assertEqual(self.eodr_fn_4.get_field('d2'), 'aug')
        self.assertEqual(self.eodr_fn_4.ext, '.vrt')


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2018, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


class FilenameObj(object):
    """
    Empty class which is used to hold different attributes of the filename.
    """

    def __init__(self):
        pass


class SmartFilename(object):

    """
    SmartFilename class handles file names with pre-defined field names
    and field length.
    """

    def __init__(self, fields, fields_def, ext=None, pad='-', delimiter='_', convert=False):
        """
        Define name of fields, length, pad and delimiter symbol.

        Parameters
        ----------
        fields : dict
            Name of fields (keys) and (values).
        field_def : OrderedDict
            Name of fields (keys) in right order and length (values).
        ext : str, optional
            File name extension (default: None).
        pad : str, optional
            Padding symbol (default: '-').
        delimiter : str, optional
            Delimiter (default: '_')
        convert: bool, optional
            If true, decoding is applied to parts of the filename, where such an operation is available (default is False).
        """
        self.fields = fields
        self.fields_def = fields_def
        self.ext = ext
        self.delimiter = delimiter
        self.pad = pad
        self.convert = convert
        self._check_def()
        self.obj = self.__init_filename_obj()
        self.full_string = self._build_fn()

    def __init_filename_obj(self):
        """
        Initialises the class 'FilenameObj' to set all filename attributes as class variables.
        This enables an easier access to filename properties.

        Returns
        -------
        FilenameObj

        """
        filename_obj = FilenameObj()
        for key, value in self.fields.items():
            if self.convert:
                setattr(filename_obj, key, self.__decode(key, value))
            else:
                setattr(filename_obj, key, self.__encode(key, value))

        return filename_obj

    def _check_def(self):
        """
        Check if fields names and length comply with definition.

        """
        for key, value in self.fields.items():
            if key in self.fields_def:
                if not self.fields_def[key]:
                    continue
                value = self.__encode(key, value)
                if self.fields_def[key]['len'] and len(value) > self.fields_def[key]['len']:
                    raise ValueError("Length does not comply with "
                                     "definition: {:} > {:}".format(
                                         len(value), self.fields_def[key]['len']))
                if 'delim' not in self.fields_def[key].keys():
                    self.fields_def[key]['delim'] = True
            else:
                raise KeyError("Field name undefined: {:}".format(key))

    def _build_fn(self):
        """
        Build file name based on fields, padding and length.

        Returns
        -------
        filename : str
            Filled file name.

        """
        filename = ''
        for name, keys in self.fields_def.items():

            if not keys:
                continue

            length = keys['len'] if keys['len'] else 1
            delimiter = self.delimiter if keys['delim'] else ''

            if name in self.fields:
                value = self.__encode(name, self.fields[name])
                if filename == '':
                    filename = value.ljust(length, self.pad)
                else:
                    filename += delimiter + value.ljust(length, self.pad)
            else:
                if filename == '':
                    filename = self.pad * length
                else:
                    filename += delimiter + self.pad * length

        if self.ext is not None:
            filename += self.ext

        return filename

    def get_field(self, key):
        """
        Returns the value of the field with a given key.

        Parameters
        ----------
        key : str
            Name of the field.

        Returns
        -------
        str, object
            Part of the filename associated with given key. Depending on the chosen flag 'convert', it is either a str
            (convert=False) or an object.
        """

        field = self.__encode(key, self.fields[key])

        # check and reset the attribute of the object variable
        field_obj = self.__encode(key, getattr(self.obj, key))
        if field_obj and (field_obj != field):
            if self.fields_def[key]['len'] and (len(field_obj) <= self.fields_def[key]['len']):
                field = field_obj
                self[key] = field

        field = field.replace(self.pad, '')

        if self.convert:
            return self.__decode(key, field)
        else:
            return field

    def __getitem__(self, key):
        """
        Returns the value of the field with a given key.

        Parameters
        ----------
        key : str
            Name of the field.

        Returns
        -------
        str, object
            Part of the filename associated with given key. Depending on the chosen flag 'convert', it is either a str
            (convert=False) or an object. If the key can't be found in the fields definition, the method tries to return
            a property of an inherited class.
        """
        if key in self.fields_def:
            return self.get_field(key)
        elif hasattr(self, key):
            return getattr(self, key)
        else:
            raise KeyError('"{}" is neither a class variable nor a file attribute.'.format(key))

    def __setitem__(self, key, value):
        """
        Sets the value of a filename field corresponding to the given key.

        Parameters
        ----------
        key : str
            Name of the field.
        value: object
            Value of the field.

        """
        if key in self.fields_def and self.fields_def[key]:
            value = self.__encode(key, value)
            if self.fields_def[key]['len'] and (len(value) > self.fields_def[key]['len']):
                raise ValueError("Length does not comply with "
                                 "definition: {:} > {:}".format(
                                     len(value), self.fields_def[key]['len']))
            else:
                self.fields[key] = value
                value = value.replace(self.pad, '')
                if self.convert:
                    setattr(self.obj, key, self.__decode(key, value))
                else:
                    setattr(self.obj, key, value)
        else:
            raise KeyError("Field name undefined: {:}".format(key))

    def __repr__(self):
        """
        Returns the string representation of the class.

        Returns
        -------
        str
            String representation of the class.
        """
        return self._build_fn()

    def __decode(self, key, value):
        """
        Decodes a certain value (str -> object) specified by the given key if an entry 'decoder' is available.

        Parameters
        ----------
        key : str
            Name of the field.
        value: object
            Value of the field.

        Returns
        -------
        str
            Decoded or original value.
        """
        if self.fields_def[key] and 'decoder' in self.fields_def[key].keys():
            decoder = self.fields_def[key]['decoder']
            try:
                dec_value = decoder(value)
                return dec_value
            except:
                return value
        else:
            return value

    def __encode(self, key, value):
        """
        Encodes a certain value (object -> str) specified by the given key if an entry 'encoder' is available.

        Parameters
        ----------
        key : str
            Name of the field.
        value: object
            Value of the field.

        Returns
        -------
        str
            Encoded or original value.
        """
        if (not isinstance(value, str)) and self.fields_def[key] and ('encoder' in self.fields_def[key].keys()):
            encoder = self.fields_def[key]['encoder']
            try:
                enc_value = encoder(value)
                return enc_value
            except:
                return value
        else:
            return value



# Copyright (c) 2018, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
eoDR file name definition.

"""

from datetime import datetime
from collections import OrderedDict


from geopathfinder.file_naming import SmartFilename


class eoDRFilename(SmartFilename):

    """
    eoDataReaders file name definition using SmartFilename class.
    """

    def __init__(self, fields, ext='.vrt', convert=False):
        """
        Constructor of eoDRFilename class.

        Parameters
        ----------
        fields: dict
            Dictionary specifying the different parts of the filename.
        ext: str, optional
            Extension of the filename (default is '.vrt' for GDAL VRT files)
        convert: bool, optional
            If true, decoding is applied to parts of the filename, where such an operation is available (default is False).
        """
        self.dt_format = "%Y%m%dT%H%M%S"
        self.fields = fields.copy()

        fields_def = OrderedDict([
                     ('id', {'len': 12, 'delim': True}),
                     ('dt_1', {'len': 15, 'delim': True,
                                  'decoder': lambda x: self.decode_datetime(x),
                                  'encoder': lambda x: self.encode_datetime(x)}),
                     ('dt_2', {'len': 15, 'delim': True,
                                  'decoder': lambda x: self.decode_datetime(x),
                                  'encoder': lambda x: self.encode_datetime(x)}),
                     ('band', {'len': None, 'delim': True, 'encoder': lambda x: str(x)})
                    ])

        fields_def_keys = list(fields_def.keys())
        for key in fields.keys():
            if key not in fields_def_keys:
                fields_def[key] = {'len': None, 'delim': True}

        super(eoDRFilename, self).__init__(self.fields, fields_def, pad='-', ext=ext, convert=convert)

    @property
    def stime(self):
        """
        Start time.

        Returns
        -------
        datetime.datetime
            Start time.
        """
        try:
            if "-" not in self['dt_1']:
                return self.decode_datetime(self['dt_1'])
            else:
                return None
        except TypeError:
            return None

    @property
    def etime(self):
        """"
        End time.

        Returns
        -------
        datetime.datetime
            End time.
        """
        try:
            if "-" not in self['dt_2']:
                return self.decode_datetime(self['dt_2'])
            else:
                return None
        except TypeError:
            return None

    def decode_datetime(self, string):
        """
        Decodes a string into a datetime object. The format is given by the class.

        Parameters
        ----------
        string: str, object
            String needed to be decoded to a datetime object.

        Returns
        -------
        datetime.datetime, object
            Original object or datetime object parsed from the given string.
        """
        if isinstance(string, str):
            return datetime.strptime(string, self.dt_format)
        else:
            return string

    def encode_datetime(self, time_obj):
        """
        Encodes a datetime object into a string. The format is given by the class.

        Parameters
        ----------
        time_obj: datetime.datetime, object
            Datetime object needed to be encoded to a string.

        Returns
        -------
        str, object
            Original object or str object parsed from the given datetime object.
        """
        if isinstance(time_obj, datetime):
            return time_obj.strftime(self.dt_format)
        else:
            return time_obj


def create_eodr_filename(filename_string, convert=False):
    """
    Creates a eoDRFilename() object from a given string filename

    Parameters
    ----------
    filename_string : str
        filename following the eoDR filename convention.
        e.g. 'c29cfaab80e2_20170517T171434_---------------_eo_array_contains.vrt'
    convert: bool, optional
            If true, decoding is applied to parts of the filename, where such an operation is available (default is False).

    Returns
    -------
    eoDRFilename
    """
    helper = eoDRFilename({})
    filename_string = filename_string.replace(helper.ext, '')
    parts = filename_string.split(helper.delimiter)

    fields = {'id': parts[0],
              'dt_1': parts[1],
              'dt_2': parts[2],
              'band': parts[3]
             }

    if len(parts) > 4:  # if the filename consists of more than 4 parts, additional "dimensions" are added to the fields dictionary
        for i, part in enumerate(parts[4:]):
            key = 'd' + str(i+1)
            fields[key] = part

    return eoDRFilename(fields, convert=convert)


if __name__ == '__main__':
    pass


# Copyright (c) 2018, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Module managing folder tree structure.
"""

import os
from functools import reduce


def get_directory_structure(root_path):
    """
    Creates a nested dictionary that represents the folder structure
    of root path.

    Parameters
    ----------
    root_path : str
        Root path.

    Returns
    -------
    dir_dict : dict
        Directory tree stored in nested dictionary.
    """
    dir_dict = {}
    root_path = root_path.rstrip(os.sep)
    start = root_path.rfind(os.sep) + 1

    for path, dirs, files in os.walk(root_path):
        folders = path[start:].split(os.sep)
        subdir = dict.fromkeys(files)
        parent = reduce(dict.get, folders[:-1], dir_dict)
        parent[folders[-1]] = subdir

    return dir_dict


def nested_lookup(key, document, wild=False):
    """
    Lookup a key in a nested document, yield a value.

    Parameters
    ----------
    key : str
        Key string.
    document : dict
        Nested dictionary.
    wild : bool, optional
        If True case insensitive key lookup (default: False).

    Returns
    -------
    result : dict
        Yielding files/folder results.
    """
    if isinstance(document, list):
        for d in document:
            for result in nested_lookup(key, d, wild=wild):
                yield result

    if isinstance(document, dict):
        for k, v in document.items():
            if key == k or (wild and key.lower() in k.lower()):
                yield v
            elif isinstance(v, dict):
                for result in nested_lookup(key, v, wild=wild):
                    yield result
            elif isinstance(v, list):
                for d in v:
                    for result in nested_lookup(key, d, wild=wild):
                        yield result


# def flatten_dict(d):
#     def items():
#         for key, value in d.items():
#             if isinstance(value, dict):
#                 for subkey, subvalue in flatten_dict(value).items():
#                     yield key + "." + subkey, subvalue
#             else:
#                 yield key, value
#     return dict(items())


class SmartTree():
    """
    Class managing folder tree.

    Parameters
    ----------
    root_path : str
        Root path.
    """

    def __init__(self, root_path):
        self.dir_dict = get_directory_structure(root_path)

    def find_key(self, key, wild=False):
        """
        Find string in directory structure.

        Parameters
        ----------
        key : str
             Search string.

        Returns
        -------
        result : list of dict
            List of files/folders.
        """
        return list(nested_lookup(key, self.dir_dict, wild=wild))

    def iter_find_key(self, key, wild=False):
        """
        Lookup a key in a nested document, return a list of values.

        Parameters
        ----------
        key : str
            Key string.
        document : dict
            Dictionary.
        wild : bool, optional
            Wild (default: False).

        Returns
        -------
        result : list of dict
            List of files/folders.
        """
        return nested_lookup(key, self.dir_dict)

    def make_tree(self, tree_dict):
        """
        Make directory structure.

        Parameters
        ----------
        tree_dict : dict
            Nested dictionary defining folder tree.
        """
        pass


def test_smarttree():
    """
    Simple SmartTree test.
    """
    root_path = os.path.join('/home', 'shahn', 'shahn',
                             'home', 'documents', 'shahn')

    stree = SmartTree(root_path)
    result = stree.find_key('test')
    print(result)


if __name__ == '__main__':
    test_smarttree()



# Copyright (c) 2018, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
import unittest
from datetime import datetime
from datetime import time

import logging

from geopathfinder.naming_conventions.sgrt_naming import SgrtFilename
from geopathfinder.naming_conventions.sgrt_naming import sgrt_tree
from geopathfinder.naming_conventions.sgrt_naming import sgrt_path
from geopathfinder.naming_conventions.sgrt_naming import create_sgrt_filename

logging.basicConfig(level=logging.INFO)


class TestSgrtFilename(unittest.TestCase):

    def setUp(self):
        self.dtime_1 = datetime(2008, 1, 1, 12, 23, 33)
        self.dtime_2 = datetime(2009, 2, 2, 22, 1, 1)

        fields = {'dtime_1': self.dtime_1, 'dtime_2': self.dtime_2,
                  'var_name': 'SSM'}

        self.sgrt_fn = SgrtFilename(fields, convert=True)

        fields = {'dtime_1': self.dtime_1,
                  'var_name': 'SSM'}

        self.sgrt_fn2 = SgrtFilename(fields, convert=True)

        self.sgrt_fn3 = SgrtFilename(fields)

        fn = 'M20170725_165004--_SIG0-----_S1BIWGRDH1VVA_146_A0104_EU500M_E048N012T6.tif'
        self.sgrt_fn4 = create_sgrt_filename(fn, convert=True)

        fn = 'M20170725_20181225_TMENSIG40_ASAWS---M1--D_146_A0104_EU500M_E048N012T6.tif'
        self.sgrt_fn5 = create_sgrt_filename(fn)

    def test1_build_sgrt_filename(self):
        """
        Test building SGRT file name.

        """
        fn = ('-20080101_20090202_SSM------_-------------_---_-----_------_----------.tif')

        self.assertEqual(self.sgrt_fn.__repr__(), fn)

    def test2_get_n_set_date(self):
        """
        Test set and get start and end date.

        """
        self.assertEqual(self.sgrt_fn['dtime_1'], self.dtime_1.date())
        self.assertEqual(self.sgrt_fn['dtime_2'], self.dtime_2.date())

        new_start_time = datetime(2009, 1, 1, 12, 23, 33).date()
        self.sgrt_fn['dtime_1'] = new_start_time

        self.assertEqual(self.sgrt_fn['dtime_1'], new_start_time)

    def test30_get_n_set_date_n_time(self):
        """
        Test set and get date and time for a single datetime.

        """
        self.assertEqual(self.sgrt_fn2['dtime_1'], self.dtime_1.date())
        self.assertEqual(self.sgrt_fn2['dtime_2'], self.dtime_1.time())

        new_start_time = datetime(2009, 1, 1, 12, 23, 33)
        self.sgrt_fn2['dtime_1'] = new_start_time
        self.sgrt_fn2['dtime_2'] = new_start_time

        self.assertEqual(self.sgrt_fn2['dtime_1'], new_start_time.date())
        self.assertEqual(self.sgrt_fn2['dtime_2'], new_start_time.time())

    def test31_get_n_set_date_n_time_dts(self):
        """
        Test get and set date and time for a single datetime,
        returning strings.

        """
        self.assertEqual(self.sgrt_fn2.obj.dtime_1, datetime(2008, 1, 1).date())
        self.assertEqual(self.sgrt_fn2.obj.dtime_2, time(12, 23, 33))

        new_start_time = datetime(2345, 1, 2, 7, 8, 9)
        self.sgrt_fn2['dtime_1'] = new_start_time
        self.sgrt_fn2['dtime_2'] = new_start_time

        self.assertEqual(self.sgrt_fn2.obj.dtime_1, datetime(2345, 1, 2).date())
        self.assertEqual(self.sgrt_fn2.obj.dtime_2, time(7, 8, 9))

    def test32_get_n_set_date_n_time_strings(self):
        """
        Test get and set date and time for a single datetime,
        returning strings.

        """
        self.assertEqual(self.sgrt_fn3.obj.dtime_1, '20080101')
        self.assertEqual(self.sgrt_fn3.obj.dtime_2, '122333')

        new_start_time = datetime(2345, 1, 2, 7, 8, 9)
        self.sgrt_fn3['dtime_1'] = new_start_time
        self.sgrt_fn3['dtime_2'] = new_start_time

        self.assertEqual(self.sgrt_fn3.obj.dtime_1, '23450102')
        self.assertEqual(self.sgrt_fn3.obj.dtime_2, '070809')

    def test4_create_sgrt_filename(self):
        """
        Tests the creation of a SmartFilename from a given string filename.

        """

        # testing for single datetime
        self.assertEqual(self.sgrt_fn4.get_field('pflag'), 'M')
        self.assertEqual(self.sgrt_fn4.get_field('dtime_1'), datetime(2017, 7, 25).date())
        self.assertEqual(self.sgrt_fn4.get_field('dtime_2'), time(16, 50, 4))
        self.assertEqual(self.sgrt_fn4.get_field('var_name'), 'SIG0')
        self.assertEqual(self.sgrt_fn4.get_field('mission_id'), 'S1')
        self.assertEqual(self.sgrt_fn4.get_field('spacecraft_id'), 'B')
        self.assertEqual(self.sgrt_fn4.get_field('mode_id'), 'IW')
        self.assertEqual(self.sgrt_fn4.get_field('product_type'), 'GRD')
        self.assertEqual(self.sgrt_fn4.get_field('res_class'), 'H')
        self.assertEqual(self.sgrt_fn4.get_field('level'), '1')
        self.assertEqual(self.sgrt_fn4.get_field('pol'), 'VV')
        self.assertEqual(self.sgrt_fn4.get_field('orbit_direction'), 'A')
        self.assertEqual(self.sgrt_fn4.get_field('relative_orbit'), 146)
        self.assertEqual(self.sgrt_fn4.get_field('workflow_id'), 'A0104')
        self.assertEqual(self.sgrt_fn4.get_field('grid_name'), 'EU500M')
        self.assertEqual(self.sgrt_fn4.get_field('tile_name'), 'E048N012T6')
        self.assertEqual(self.sgrt_fn4.ext, '.tif')

        # testing for empty fields and two dates
        self.assertEqual(self.sgrt_fn5.get_field('dtime_1'), '20170725')
        self.assertEqual(self.sgrt_fn5.get_field('dtime_2'), '20181225')
        self.assertEqual(self.sgrt_fn5.get_field('var_name'), 'TMENSIG40')
        self.assertEqual(self.sgrt_fn5.get_field('mission_id'), 'AS')
        self.assertEqual(self.sgrt_fn5.get_field('spacecraft_id'), 'A')
        self.assertEqual(self.sgrt_fn5.get_field('mode_id'), 'WS')
        self.assertEqual(self.sgrt_fn5.get_field('product_type'), '')
        self.assertEqual(self.sgrt_fn5.get_field('res_class'), 'M')
        self.assertEqual(self.sgrt_fn5.get_field('level'), '1')
        self.assertEqual(self.sgrt_fn5.get_field('pol'), '')

    def test5_build_ascat_ssm_fname(self):
        """
        Tests the creation of a resampled ASCAT SSM filename.

        """
        date_time = '20331122_112233'
        tilename = 'EU500M_E012N024T6'

        xfields = {'pflag': 'D',
                   'dtime_1': datetime.strptime(date_time[:8], "%Y%m%d"),
                   'dtime_2': datetime.strptime(date_time[-6:], "%H%M%S"),
                   'var_name': 'SSM',
                   'mission_id': 'AS',
                   'spacecraft_id': 'C',
                   'mode_id': 'SM',
                   'product_type': 'O12',
                   'res_class': 'N',
                   'level': 'A',
                   'pol': 'XX',
                   'orbit_direction': 'D',
                   'workflow_id': 'C0102',
                   'grid_name': tilename[:6],
                   'tile_name': tilename[7:]}

        should = 'D20331122_112233--_SSM------_ASCSMO12NAXXD_---_C0102_EU500M_E012N024T6.tif'
        fn = SgrtFilename(xfields)
        self.assertEqual(fn.full_string, should)


class TestSgrtPath(unittest.TestCase):
    """
    Tests checking if a SGRT path is correctly reflected by sgrt_tree.
    """

    def setUp(self):
        """
        Setting up the test sgrt_tree.
        """

        self.test_dir = os.path.join(os.path.dirname(
            os.path.abspath(__file__)), 'test_data', 'Sentinel-1_CSAR')

    def test_full_path(self):
        """
        Tests the SmartPath() for the SGRT naming conventions
        """

        should = os.path.join(self.test_dir, 'IWGRDH', 'products', 'datasets',
                              'ssm', 'C1003', 'EQUI7_EU500M', 'E048N012T6',
                              'ssm', 'qlooks')

        stp1 = sgrt_path(self.test_dir,
                         mode='IWGRDH', group='products', datalog='datasets',
                         product='ssm', wflow='C1003', grid='EQUI7_EU500M',
                         tile='E048N012T6', var='ssm',
                         qlook=True, make_dir=False)

        self.assertEqual(stp1.directory, should)

        # giving no specifications on group and datalog levels
        stp2 = sgrt_path(self.test_dir,
                         mode='IWGRDH', product='ssm', wflow='C1003',
                         grid='EQUI7_EU500M', tile='E048N012T6', var='ssm',
                         qlook=True, make_dir=False)

        self.assertEqual(stp2.directory, should)

        pass


class TestSgrtTree(unittest.TestCase):
    """
    Tests checking if a SGRT tree is correctly reflected by sgrt_tree.
    """

    def setUp(self):
        """
        Setting up the test sgrt_tree.
        """

        self.test_dir = os.path.join(os.path.dirname(
            os.path.abspath(__file__)), 'test_data', 'Sentinel-1_CSAR')
        self.hierarchy_should = ['root', 'mode', 'group', 'datalog', 'product',
                                 'wflow', 'grid', 'tile', 'var', 'qlook']
        self.stt = sgrt_tree(self.test_dir, register_file_pattern='.tif')


    def test_tree_hierarchy(self):
        """
        Tests if a correct SGRT hierarchy was built.
        """

        self.assertEqual(self.stt.hierarchy, self.hierarchy_should)

        self.assertEqual(self.stt.root, self.test_dir)


    def test_tree_depth(self):
        """
        Checks if maximum depth is not violated.
        """

        max_depth_allowed = len(self.stt.root.split(os.sep)) + \
                            len(self.hierarchy_should) - 1

        self.assertTrue(all([len(x.split(os.sep)) <= max_depth_allowed
                             for x in self.stt.get_all_dirs()]))


if __name__ == "__main__":
    unittest.main()
